package eu.etaxonomy.taxeditor;


import org.eclipse.core.runtime.IStatus;
import org.eclipse.ui.application.IWorkbenchConfigurer;
import org.eclipse.ui.application.IWorkbenchWindowConfigurer;
import org.eclipse.ui.application.WorkbenchAdvisor;
import org.eclipse.ui.application.WorkbenchWindowAdvisor;
import org.eclipse.ui.statushandlers.AbstractStatusHandler;
import org.eclipse.ui.statushandlers.StatusAdapter;
import org.springframework.remoting.RemoteAccessException;
import org.springframework.remoting.RemoteConnectFailureException;

import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;



/**
 * <p>ApplicationWorkbenchAdvisor class.</p>
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public class ApplicationWorkbenchAdvisor extends WorkbenchAdvisor {

	private CdmStatusHandler cdmStatusHandler;

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.application.WorkbenchAdvisor#createWorkbenchWindowAdvisor(org.eclipse.ui.application.IWorkbenchWindowConfigurer)
	 */
	/** {@inheritDoc} */
	@Override
    public WorkbenchWindowAdvisor createWorkbenchWindowAdvisor(
			IWorkbenchWindowConfigurer configurer) {
		return new ApplicationWorkbenchWindowAdvisor(configurer);
	}


	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.application.WorkbenchAdvisor#getInitialWindowPerspectiveId()
	 */
	/**
	 * <p>getInitialWindowPerspectiveId</p>
	 *
	 * @return a {@link java.lang.String} object.
	 */
	@Override
    public String getInitialWindowPerspectiveId() {
		return "eu.etaxonomy.taxeditor.application.perspective.taxonomic";
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.application.WorkbenchAdvisor#initialize(org.eclipse.ui.application.IWorkbenchConfigurer)
	 */
	/** {@inheritDoc} */
	@Override
    public void initialize(IWorkbenchConfigurer configurer) {
		super.initialize(configurer);

		// Remembers the user's view layout, window size, window location etc.
		//  for the next time application is started
		configurer.setSaveAndRestore(true);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.application.WorkbenchAdvisor#getWorkbenchErrorHandler()
	 */
	@Override
	public synchronized AbstractStatusHandler getWorkbenchErrorHandler() {
	    if (cdmStatusHandler == null) {
	        cdmStatusHandler = new CdmStatusHandler();
	    }
	    return cdmStatusHandler;
	}


	/**
	 * Custom status handler for handling scenarios which are
	 * not handled by the editor (e.g. runtime exceptions).
	 *
	 * The default {@link org.eclipse.ui.statushandlers.WorkbenchErrorHandler}
	 * is not used or extended because we need a handler for specific scenarios
	 * which displays a custom built error dialog.
	 *
	 * @author cmathew
	 *
	 */
	class CdmStatusHandler extends AbstractStatusHandler {

	    private Throwable previousT;
		/* (non-Javadoc)
		 * @see org.eclipse.ui.statushandlers.AbstractStatusHandler#handle(org.eclipse.ui.statushandlers.StatusAdapter, int)
		 */
		@Override
		public void handle(StatusAdapter statusAdapter, int style)
		{

		    if(statusAdapter.getStatus().matches(IStatus.ERROR)) {

		    	IStatus status = statusAdapter.getStatus();
		    	Throwable t = statusAdapter.getStatus().getException();
		    	// NOTE : the global status handling mechanism in the case of
		    	//        runtime exceptions is called twice, once by the application
		    	//        throwing the exception and then by the rcp logging mechanism
		    	//        The check below is to make sure that the same exception is
		    	//        not shown twice in succession.
		    	if(t != null && previousT == t) {
	                return;
	            }
		    	previousT = t;



		    	// NOTE : Currently we only allow RuntimeExceptions since
		    	//        allowing all kinds of exceptions would also include
		    	//        those in generated status objects coming from from logging triggers
		    	//        leading to a recursive infinite loop of :
		    	//        initial exception thrown -> status handling -> dialog opening + logging of status ->
		    	//        status handling -> dialog opening + logging of status ... and so on
		    	if(t != null &&
		    	        t instanceof RuntimeException &&
		    	        ! "Widget is disposed".equals(t.getMessage()) &&
		    	        ! handleKnownRuntimeException(t,statusAdapter.getStatus().getPlugin())) {

		    	    MessagingUtils.errorDialog("Unexpected error",
		    	            null,
		    	            MessagingUtils.UNEXPECTED_ERROR_MESSAGE,
		    	            statusAdapter.getStatus().getPlugin(),
		    	            t,
		    	            true);

		    	} else if (t != null && ("Widget is disposed".equals(t.getMessage()))){
                    MessagingUtils.warn(this.getClass(), t);
                    if (PreferencesUtil.isShowUpWidgetIsDisposedMessages()){
                        MessagingUtils.errorDialog("Widget is disposed",
                                null,
                                MessagingUtils.WIDGET_IS_DISPOSED_MESSAGE,
                                statusAdapter.getStatus().getPlugin(),
                                t,
                                true);

                    }
                }
		    }
		}

		private boolean handleKnownRuntimeException(Throwable t, String pluginId) {
		    if(t instanceof RemoteConnectFailureException ||
		            t.getCause() instanceof RemoteConnectFailureException) {
		        MessagingUtils.errorDialog("Connection Failure",
		                null,
		                MessagingUtils.CONNECTION_FAILURE_MESSAGE + System.getProperty("line.separator"),
		                pluginId,
		                t,
		                true,
		                false);
		        return true;
		    }
		    if(t instanceof RemoteAccessException ||
		            t.getCause() instanceof RemoteAccessException) {
		        MessagingUtils.errorDialog("Remote Access Error",
		                null,
		                MessagingUtils.REMOTE_ACCESS_FAILURE_MESSAGE + System.getProperty("line.separator"),
		                pluginId,
		                t,
		                true,
		                false);
		        return true;
		    }
		    return false;
		}
	}



}
